﻿Checklist2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "AceEvent-2.0")
Checklist2.name = "Checklist2"
--[[ Moved into their respective metatags in the toc file
Checklist2.credits = "Everyone who posted their suggestions on Curse and WoWI :D"
Checklist2.date = "Saturday February 9, 2008 8:54 AM GMT+8"
Checklist2.email = "quezpwns AT gmail DOT com"

--$Rev: 65594 $
--$Date: 2008-03-24 22:10:12 -0400 (Mon, 24 Mar 2008) $
]]
local L = AceLibrary("AceLocale-2.2"):new("Checklist2")

-- Used for the update checks performed on login (not yet written)
-- YYYYMMDDTTTT
-- Year, Month, Date, Time (24h format)
local INTERNAL_VERSION_ID   = 200803251003
Checklist2.version = "2.0.0.0725"
local LISTBAR_ALWAYS_SHOWN  = true
local ITEMBAR_ALWAYS_SHOWN  = true
local ListInUse
local activeChecks
local activeItems


Checklist2.versionID = INTERNAL_VERSION_ID;

-- Bindings
BINDING_HEADER_CHECKLIST2 = "Checklist2"
BINDING_NAME_SHOW_CHECKLIST2_UI = L["Toggle Checklist2 window"]

-- Ace2 slash command definitions
local consoleoptions = {
    type = "group",
    desc = L["A checklist to keep track of tasks you need to do, now rewritten to include more features."],
    args = {
    ui = {
      name = "UI",
      type = 'execute',
      desc = L["Toggles the Checklist2 window"],
      func = function()
        Checklist2:ToggleUI()
      end,
      },
    reset = {
      name = L["Reset"],
      type = 'execute',
      desc = L["Resets the Checklist2 window's position, then shows it"],
      func = function()
        Checklist2_CoreFrame:ClearAllPoints()
        Checklist2_CoreFrame:SetPoint("CENTER", "UIParent", "CENTER")
        Checklist2SetupFrame:ClearAllPoints()
        Checklist2SetupFrame:SetPoint("CENTER", "Checklist2_CoreFrame", "CENTER")
        Checklist2_CoreFrame:Show()
        Checklist2:Print(L["Window positions have been reset."])
      end
      },
    newlist = {
      name = "New List",
      type = "text",
      desc = L["Creates a new empty list, ready for customization"],
      usage = "|cffffff00listname|r",
      get = false,
      set = function(listname)
        Checklist2:CreateList(listname)
        Checklist2:ListScrollBarUpdate()
      end
      },
    renamelist = {
      name = "Rename List",
      type = "text",
      desc = L["Change the name of the specified list"],
      usage = "|cffffff00old name, new name|r",
      get = false,
      set = function(val)
        local oldName, newName = strsplit(', ', val, 2)
        if (oldName and newName) then
          Checklist2:RenameList(oldName, newName)
        else
          self:Print(L["Unable to parse parameters to |cffffff00/checklist2 renamelist|r, please try again"])
        end
      end
    },
    deletelist = {
      name = "Delete List",
      type = "text",
      desc = L["Delete the specified list."],
      usage = "|cffffff00listname|r",
      get = false,
      set = function(listname)
        if listname == Checklist2:GetListInUse(true) then
          Checklist2:Print(format(L["Cannot delete the currently displayed list (|cffffff00%s|r)"], listname))
          return
        end
        if Checklist2:GetList(listname) then
          Checklist2.Delete = listname
          StaticPopup_Show("CHECKLIST2_DELETE_LIST", Checklist2.Delete)
        else
          Checklist2:Print("Could not find list \"|cffffff00"..listname.."|r\"")
        end
      end
      },
    setlist = {
      name = "setlist",
      type = "text",
      desc = L["Sets the list to display on the plugin tooltip and the main window."],
      usage = "|cffffff00listname|r",
      get = false,
      set = function(listname)
        Checklist2:SetListInUse(listname)
        Checklist2Fu:Update()
      end
    },
      check = {
        name = "Check",
        type = "text",
        desc = L["Checks the specified task number"],
        usage = L["|cffffff00tasknumber|r\n(only accepts numbers)"],
        get = false,
        set = function(taskID)
          taskID = tonumber(taskID)
          if not taskID then
            Checklist2:Print(L["Value given into |cffffff00/checklist2 check|r cannot be converted into a number."])
            return
          end
          taskID = math.floor(taskID)
          if taskID > 0 and taskID <= Checklist2:GetNumItems() then
            Checklist2:GetListInUse().checks["checked"..taskID] = true
            Checklist2:ListItemSBUpdate()
            Checklist2Fu:Update()
          else
            Checklist2:Print(format(L["The specified task (|cffffff00%s|r) was not found."], taskID))
          end
        end
      },
      uncheck = {
        name = "Check",
        type = "text",
        desc = L["Unchecks the specified task number"],
        usage = L["|cffffff00tasknumber|r\n(only accepts numbers)"],
        get = false,
        set = function(taskID)
          taskID = tonumber(taskID)
          if not taskID then
            Checklist2:Print(L["Value given into |cffffff00/checklist2 uncheck|r cannot be converted into a number."])
            return
          end
          taskID = math.floor(taskID)
          if taskID > 0 and taskID <= Checklist2:GetNumItems() then
            Checklist2:GetListInUse().checks["checked"..taskID] = false
            Checklist2:ListItemSBUpdate()
            Checklist2Fu:Update()
          else
            Checklist2:Print(format(L["The specified task (|cffffff00%s|r) was not found."], taskID))
          end
        end
      }
      --[[,
    memory = {
      name = "Memory",
      type = 'execute',
      desc = "Show how much memory Checklist2 is currently using.",
      func = function()
        UpdateAddOnMemoryUsage()
        Checklist2:Print("Memory usage is currently at |cFF7FFF7F"..GetAddOnMemoryUsage("Checklist2").." KiB|r")
      end
      }]]
    }
  }

Checklist2:RegisterChatCommand({"/checklist2", "/cl2"}, consoleoptions)

function Checklist2:OnInitialize()
  -- Change the font
  for i = 1, 15 do
    local itemtext = getglobal("Checklist2_CoreFrameCheckButton"..i.."Text")
    itemtext:SetFont("Fonts\\FRIZQT__.TTF", 12, " ")
    itemtext:SetWidth(645)
    itemtext:SetJustifyH("LEFT")
  end
  --self:Print("Checklist2 online!")
  if self.db.profile.Settings.latestKnownVersionID > INTERNAL_VERSION_ID then  -- Check if an update has been detected earlier
    local text
    if self.db.profile.Settings.latestKnownVersionString then
      text = "(|cff7fff7f"..self.db.profile.Settings.latestKnownVersionString.."|r)"
    else
      text = ""
    end
    self:Print(L["A newer version is available. "]..text)
  end
  if self.db.profile.Settings.latestKnownVersionID < INTERNAL_VERSION_ID then  -- The user updated to a version higher than has been detected
    self.db.profile.Settings.latestKnownVersionID = INTERNAL_VERSION_ID
    self.db.profile.Settings.latestKnownVersionString = self.version
  end
  Checklist2_CoreFrameTexture1Texture:SetAlpha(.35)
  if self:GetNumLists() == 0 then
    self:CreateHelpList()
  end
  self:SetListInUse(Checklist2.db.profile.Settings.ListLastUsed)
  self:CollectListNames()
  self:SetCommPrefix("Checklist2")
  self.online = false
  if not self.db.profile.Settings.versionID then
    self.db.profile.Settings.versionID = 0;
  end
  Checklist2_CoreFrameAddRemoveItemsButtonFS:SetFormattedText(L["Tasks: %i/%i (%s)"], self:GetNumTasksChecked(), self:GetNumItems(), self:GetPercentageChecked())
end

function Checklist2:OnEnable()
  self:RegisterComm("Checklist2", "GROUP")
  -- "GROUP" distribution chooses between BATTLEGROUND, PARTY and RAID, depending
  -- on which is most applicable.
  -- Received messages are passed into method OnCommReceive()
  self:RegisterComm("Checklist2", "WHISPER")
  self:RegisterComm("Checklist2", "GUILD")
end

function Checklist2:OnProfileEnable()
  -- Update the UI
  if self:GetNumLists() == 0 then
    self:CreateHelpList()
  end
  self:SetListInUse(Checklist2.db.profile.Settings.ListLastUsed)
  self:CollectListNames()
  self:ListScrollBarUpdate()
  self:ListItemSBUpdate()
end

-- Checklist2 core functions
function Checklist2:ToggleUI()
  if Checklist2SetupFrame:IsVisible() then
    return
  end
  if not Checklist2_CoreFrame:IsVisible() then
    Checklist2_CoreFrame:Show()
    UIFrameFadeIn(Checklist2_CoreFrame, 0.075, 0, 1)
  else
    -- Cannot fade out, hides frame immediately :(
    Checklist2_CoreFrame:Hide()
  end
end



-- Revamped SetListInUse method
-- List names are now used as identifiers instead of a numeric ID
function Checklist2:SetListInUse(listName, override)
  if listName == ListInUse and not override then
    return  -- Save ourselves the effort
  end
  -- ChatFrame1:AddMessage(tostring(listName))
  if Checklist2SetupFrame:IsVisible() then
    --self:Print("Cannot change lists while the setup dialog is open.")
    return false
  end
  if type(listName) ~= "string" then
    error("Invalid argument passed into Checklist2 method SetListInUse (expecting string, got "..type(listName)..")")
  end
  if not self:GetList(listName) then  -- Safe failure
    self:Print(format(L["The list \"%s\" was not found."], listName))
    return
  end
  activeChecks = self:GetList(listName).checks
  activeItems = self:GetList(listName).items
  ListInUse = listName
  self:ListItemSBUpdate()
  self.db.profile.Settings.ListLastUsed = listName
  Checklist2_CoreFrameLabel1Label:Show()
  Checklist2_CoreFrameLabel1Label:SetText(listName.." - Checklist v"..self.version)
  self:ListScrollBarUpdate()
  Checklist2_CoreFrameAddRemoveItemsButtonFS:SetText(format(L["Tasks: %i/%i (%s)"], self:GetNumTasksChecked(), self:GetNumItems(), self:GetPercentageChecked()))
end

function Checklist2:GetNumItems()
  local numItems = 0
  for k,v in pairs(self:GetListInUse().items) do
    numItems = numItems + 1
  end
  return numItems
end


function Checklist2:GetNumLists()
  local lists = 0
  local liststbl = self.db.profile.Lists
  for k,v in pairs(liststbl) do
    lists = lists + 1
  end
  return lists
end

function Checklist2:GetListInUse(giveName)
  if giveName then
    return ListInUse
  end
  return self:GetList(ListInUse)
end

function Checklist2:GetList(list)
  return self.db.profile.Lists[list]
end


function Checklist2:CheckAll()
  local _,id
  for k,v in pairs(self:GetListInUse().checks) do
    _,_,id = strfind(k, "checked(.+)")
    if not v then -- if unchecked
      Checklist2:CheckClicked(id, true)
    end
  end
  self:ListItemSBUpdate()
  Checklist2Fu:Update()
end

function Checklist2:UncheckAll()
  local _,id
  for k,v in pairs(self:GetListInUse().checks) do
    _,_,id = strfind(k, "checked(.+)")
    if v then -- if checked
      Checklist2:CheckClicked(id, false)
    end
  end
  self:ListItemSBUpdate()
  Checklist2Fu:Update()
end

function Checklist2:CheckClicked(checkID, isChecked)
  -- convert nil to false, 1 to true
  -- workaround for a bug where only a portion of a list gets modified when
  -- "Check All" or "Uncheck All" gets clicked
  -- By the way, this is bad grammar if used in English (double negatives :) )
  self:GetListInUse().checks["checked"..tostring(checkID)] = not (not isChecked)
  Checklist2_CoreFrameAddRemoveItemsButtonFS:SetText(format(L["Tasks: %i/%i (%s)"], self:GetNumTasksChecked(), self:GetNumItems(), self:GetPercentageChecked()))
end

function Checklist2:GetPercentageChecked()
  local numTasks, numChecked = 0, 0
  for k,v in pairs(self:GetListInUse().checks) do
    numTasks = numTasks + 1
    if v then
      numChecked = numChecked + 1
    end
  end
  local percChecked = (numChecked / numTasks) * 100
  percChecked = (floor(percChecked*10)/10)
  percChecked = percChecked.."%"
  return percChecked
end

-- List menu GUI update function
function Checklist2:ListScrollBarUpdate()
  if not self.listNames then
    self:CollectListNames()
  end
  local line;
  local lineplusoffset;
  local names;
  local text;
  FauxScrollFrame_Update(Checklist2AppletScrollBar, Checklist2:GetNumLists(), 30, 16, nil, nil, nil, nil, nil, nil, LISTBAR_ALWAYS_SHOWN);
  for line = 1,30 do
    local button = getglobal("Checklist2AppletListContainerButton"..line)
    lineplusoffset = line + FauxScrollFrame_GetOffset(Checklist2AppletScrollBar);
    if lineplusoffset <= Checklist2:GetNumLists() then
      text = self.listNames[lineplusoffset]
      button:SetTextColor(1,1,1)
      if self.listNames[lineplusoffset] == ListInUse then
        button:SetTextColor(1,1,0)
      end
      button:SetText(text);
      button.list = self.listNames[lineplusoffset];
      button:Show();
    else
      button:Hide();
    end
  end
end

-- List item GUI updater function
function Checklist2:ListItemSBUpdate()
  local numberOfItems = self:GetNumItems();
  local line;
  local lineplusoffset;
  FauxScrollFrame_Update(Checklist2ChecksScrollBar, numberOfItems, 15, 32, nil, nil, nil, nil, nil, nil, ITEMBAR_ALWAYS_SHOWN);
  for line=1,15 do
    local listItem = getglobal("Checklist2_CoreFrameCheckButton"..tostring(line));
    local listItemText = getglobal(listItem:GetName().."Text");
    local setButton = getglobal("Checklist2SetButton"..line);
    lineplusoffset = line + FauxScrollFrame_GetOffset(Checklist2ChecksScrollBar);
    if lineplusoffset <= numberOfItems then
      setButton:SetText(tostring(lineplusoffset)..".");
      setButton:Show();
      setButton:SetID(lineplusoffset);
      listItemText:SetText(activeItems["check"..lineplusoffset.."Text"]);
      listItem:SetChecked(self:GetListInUse().checks["checked"..lineplusoffset]);
      listItem:SetID(lineplusoffset);
      listItem:Show();
    else
      setButton:Hide();
      listItem:Hide();
    end
  end
end

function Checklist2:CreateList(listName, norand)
  if (type(listName) ~= "string")  then
    error("Invalid argument passed into Checklist2 method \"CreateList\" (expecting string, got "..type(listName)..")")
  end
  if not norand then
    self.db.profile.Lists[listName] = self:GenerateRandomList()
  else
    self.db.profile.Lists[listName] = {
      checks = { checked1   = false },
      items  = { check1Text = self.Sayings[random(1, #self.Sayings)] }
    }
  end
  self:CollectListNames()
  self:ListScrollBarUpdate()
end

function Checklist2:CreateListItem(dontUpdateGUI)
  local x = self:GetNumItems() + 1
  self:GetListInUse().checks["checked"..x] = false
  self:GetListInUse().items["check"..x.."Text"] = self.Sayings[random(1, #self.Sayings)]
  if dontUpdateGUI then
    -- For create list items "en masse"
    -- Updating the GUI is slow, so it's best to skip it when this
    -- function will get called repeatedly.
    return
  end
  self:ListItemSBUpdate()
  Checklist2Fu:Update()
end

function Checklist2:ReduceListItems(dontUpdateGUI)
  local x = self:GetNumItems()
  if x > 1 then
    self:GetListInUse().checks["checked"..x] = nil
    self:GetListInUse().items["check"..x.."Text"] = nil
    if not dontUpdateGUI then
      self:ListItemSBUpdate()
      Checklist2Fu:Update()
    end
  else
    self:Print(L["There is only one list item left!"])
  end
end

function Checklist2:ReduceListItemsEnMasse(num)
  local x = self:GetNumItems()
  num = tonumber(num)
  if not num then
    self:Print(L["Entered value cannot be converted into a number"])
    return
  end
  if num > 50 or num > (Checklist2:GetNumItems() - 1) then
    Checklist2:Print(L["Cannot remove more than 50 list items, or the amount of list items currently existing."])
    return
  end
  for i = 1, num do -- this shouldn't execute if it's not a number. Sanity checks above.
    self:GetListInUse().checks["checked"..tostring(x)] = nil
    self:GetListInUse().items["check"..tostring(x).."Text"] = nil
    x = x - 1
  end
  self:ListItemSBUpdate()
  Checklist2Fu:Update()
end

function Checklist2:OpenSetup(id)
  Checklist2EB_ConfirmText:SetFormattedText(L["You are editing options for list item %s in list \"%s\".\nEnter text below, or press Escape to exit."], id, ListInUse)
  Checklist2.OPEN_ITEM_ID = id
  Checklist2SetupFrameEB:SetText(Checklist2:GetListInUse().items["check"..Checklist2.OPEN_ITEM_ID.."Text"])
  Checklist2SetupFrame:Show()
end

--[[
function Checklist2:CloseSetupFrame()
  
end

function Checklist2:HideWindow()
  Checklist2_CoreFrame:Hide()
  Checklist2_CoreFrame:SetAlpha(1)
end

function Checklist2:ShowWindow()
  Checklist2_CoreFrame:Show()
end
--]]

function Checklist2:RenameList(oldName, newName)
  --[[
  if (type(oldName) ~= "string") or (type(newName) ~= "string") then
    error("Checklist2 method RenameList expects two string arguments (oldName, newName)")
  end
  --]]
  assert((type(oldName == "string") and type(newName) == "string"), "Bad argument(s) to method Checklist2:RenameList(oldName, newName)")
  if not self:GetList(oldName) then
    self:Print(L["The specified list was not found."])
  end
  if not self:GetList(newName) then
    if oldName == ListInUse then
      Checklist2_CoreFrameLabel1Label:SetText(newName.." - Checklist v"..self.version)
      ListInUse = newName;
    end
    self.db.profile.Lists[newName] = self:GetList(oldName)
    self.db.profile.Lists[oldName] = nil
    self:CollectListNames()
    self:ListScrollBarUpdate()
  else
    self:Print(format(L["A list named \"%s\" already exists."], newName))
  end
  Checklist2Fu:Update()
end

function Checklist2:DeleteList(list)
  if type(list) ~= "string" then
    error("Checklist2:DeleteList(): Expecting string, got "..type(list))
  end
  if list ~= ListInUse then
    self.db.profile.Lists[list] = nil
    self:CollectListNames()
    self:ListScrollBarUpdate()
  else
    self:Print(format(L["Cannot delete the currently displayed list (|cffffff00%s|r)"], ListInUse))
  end
end

function Checklist2:Switch(item1, item2)
  if not self:GetListInUse().items["check"..item2.."Text"] then
    self:Print(L["That task is already at either the top or the bottom of the list."])
    return
  end
  local check, item = self:GetListInUse().checks["checked"..item2], self:GetListInUse().items["check"..item2.."Text"]
  --[[ debug stuff
  self:PrintLiteral(check, item)
  self:PrintLiteral(self:GetListInUse().checks["checked"..item1], self:GetListInUse().items["check"..item1.."Text"])
  --]]
  self:GetListInUse().checks["checked"..item2] = self:GetListInUse().checks["checked"..item1]
  self:GetListInUse().items["check"..item2.."Text"]  = self:GetListInUse().items["check"..item1.."Text"]
  self:GetListInUse().checks["checked"..item1] = check
  self:GetListInUse().items["check"..item1.."Text"]  = item
  if Checklist2SetupFrame:IsVisible() then
    self:ListItemSBUpdate()
    Checklist2.OPEN_ITEM_ID = item2
    Checklist2EB_ConfirmText:SetFormattedText(L["You are editing options for list item %s in list \"%s\".\nEnter text below, or press Escape to exit."], item2, ListInUse)
  end
  Checklist2Fu:Update()
end

StaticPopupDialogs["CHECKLIST2_DELETE_LIST"] = {
  text = L["Are you sure you want to delete the list \"%s\"?"],
  button1 = TEXT(ACCEPT),
  button2 = TEXT(CANCEL),
  OnAccept = function()
      Checklist2:DeleteList(Checklist2.Delete); 
      -- .Delete is set by clicking on the "Delete" dropdown button.
  end,
  timeout = 0,
  showAlert = 1,
  whileDead = 1,
  hideOnEscape = 1
};

function Checklist2:CollectListNames()
  if not self.listNames then
    self.listNames = {}
  end
  -- This manually clears the table, allowing Checklist2 to recycle it.
  -- This method of emptying a table takes more and more time as the table
  -- gets larger.
  for i,v in ipairs(self.listNames) do
    self.listNames[i] = nil;
  end
  for k,v in pairs(self.db.profile.Lists) do
    if (type(v) == "table") then -- Check if it hasn't been deleted or tampered with.
      -- tinsert(self.listNames, k) -- Insert the table key, which is the name of the list
      self.listNames[#self.listNames + 1] = k
    end
  end
  table.sort(self.listNames) -- Alphabetize
end

function Checklist2:GetNumTasksChecked(list)
  local checks = self:GetListInUse().checks
  local numChecked = 0
  for k,v in pairs(checks) do
    if v then
      numChecked = numChecked + 1
    end
  end
  return numChecked
end

-- Debug function that shows the 30 list item container buttons.
-- Primarily used for positioning.

--[[
function cl2showAll()
  for line = 1,30 do
    getglobal("Checklist2AppletListContainerButton"..line):SetText("List Container Button "..line);
    getglobal("Checklist2AppletListContainerButton"..line):Show();
  end
end
--]]

-- Helper function
-- http://www.wowwiki.com/Tremovebyval
--[[
function Checklist2:tremovebyval(tab, val)
  for k,v in tab do
    if(v==val) then
      table.remove(tab, k);
      return true;
    end
  end
  return false;
end
--]]

--[[
function Checklist2:TabHandler(id)
  if not type(id) == "number" then 
    error("Bad arg #2 passed into :TabHandler (expecting number, got "..type(id)..")")
  end
  PlaySound("igCharacterInfoTab")
  PanelTemplates_SetTab(Checklist2_CoreFrame, id)
  self["OnTab"..id.."Click"](self)
end

function Checklist2:OnTab1Click()
  self.online = false;
  Checklist2_CoreFrameConnectingText:Hide()
  self:ShowItemsSection()
  self:CollectListNames()
  self:ListScrollBarUpdate()
end

function Checklist2:OnTab2Click()
  self.online = true;
  self:HideItemsSection()
  Checklist2_CoreFrameConnectingText:Show()
  if not self.sid then
    self:Send("get_sid")
  end
  -- Checklist2:Send("connect", "somedata", "moredata", {"somestringinatable"})
  self:CollectSharedListNames()
  self:ListScrollBarUpdate()
end

function Checklist2:OnTab3Click()
  
end

function Checklist2:OnTab4Click()
  
end
--]]

-- "Leave a message" to the other files signaling that the core loaded succesfully
Checklist2.CORE_LOADED_SUCCESSFULLY = true;